package com.juanjo.uoc.workoutshare;


import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.Typeface;

import android.os.Bundle;

import android.support.design.widget.NavigationView;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.DefaultItemAnimator;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.jjoe64.graphview.GraphView;
import com.jjoe64.graphview.LegendRenderer;
import com.jjoe64.graphview.helper.StaticLabelsFormatter;
import com.jjoe64.graphview.series.DataPoint;
import com.jjoe64.graphview.series.LineGraphSeries;


import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

public class CommonActivity extends AppCompatActivity implements NavigationView.OnNavigationItemSelectedListener {

    private DatabaseReference mDatabase;
    private DatabaseReference mWorkouts;
    private DatabaseReference mUsers;
    private DatabaseReference mBodyData;
    private DatabaseReference connectedRef;

    public static final String WORKOUTS = "WorkOuts";
    public static final String USERS = "Users";
    public static final String BODYDATA = "BodyData";
    public static final String ERROR_READ = "Error. Could not read the database";
    public static final String BUNDLE_NAME = "MY_WORKOUT";

    ArrayList<String> subKeys;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_main);

        //TOOLBAR
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayShowTitleEnabled(false);

        //FIREBASE
        databaseSetUp();

        //NAVIGATION DRAWER
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);

        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.setDrawerListener(toggle);
        toggle.syncState();
        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(this);

        //DRAWER USERNAME
        View v = navigationView.getHeaderView(0);
        TextView userTv = (TextView)v.findViewById(R.id.navdrawer_email);
        String username = getLoggedUser();
        userTv.setText(username);


    }

    /**
     * database set up needed for the future queries
     */
    protected void databaseSetUp(){
        mDatabase = FirebaseDatabase.getInstance().getReference();
        mWorkouts = mDatabase.child(WORKOUTS);
        mUsers = mDatabase.child(USERS);
        mBodyData = mDatabase.child(BODYDATA);
        connectedRef = FirebaseDatabase.getInstance().getReference(".info/connected");
    }

    /**
     * Display a Toast if we dont have connection
     */
    protected void checkConnection(){
        connectedRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot snapshot) {
                boolean connected = snapshot.getValue(Boolean.class);
                if (!connected) {
                    //Toast.makeText(getApplicationContext(), "Network problem. Not connected to firebase. Try reloading", Toast.LENGTH_SHORT).show();
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
            }
        });
    }

    /**
     * Creates a workout in database
     * @param wo
     */
    protected void create_wo(WorkOut wo){
        DatabaseReference mypostref = mWorkouts.push();
        wo.setKey(mypostref.getKey());
        mypostref.setValue(wo);
        Toast.makeText(getApplicationContext(), "WorkOut "+wo.name+" created", Toast.LENGTH_SHORT).show();

    }

    /**
     * Creates a body data register in database
     * @param bodyData
     */
    protected void create_bodyData(BodyData bodyData){
        mBodyData.push().setValue(bodyData);
        Toast.makeText(getApplicationContext(), "Body data register added", Toast.LENGTH_SHORT).show();
    }


    /**
     * Calculate and display a graph view based on body data registers
     */
    protected void load_bodyData(){
        checkConnection();
        mBodyData.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                BodyData bodyData = null;
                ArrayList<BodyData> bodyList = new ArrayList<BodyData>();
                for (DataSnapshot postSnapshot : dataSnapshot.getChildren()) {
                    BodyData auxData = postSnapshot.getValue(BodyData.class);
                    //we pick the user body datas for the graph
                    if(auxData.email.equals(getLoggedUser())) {
                        bodyList.add(auxData);
                        bodyData = auxData;
                    }
                }

                //firebase finished
                //UPDATE SOMETHING
                if(bodyData!=null){
                    //update txtviews
                    TextView tvHeight_user = (TextView) findViewById(R.id.tvHeight_user);
                    TextView tvWeight_user = (TextView) findViewById(R.id.tvWeight_user);
                    TextView tvFat_user = (TextView) findViewById(R.id.tvFat_user);
                    TextView tvMuscle_user = (TextView) findViewById(R.id.tvMuscle_user);
                    tvHeight_user.setText((Float.toString(bodyData.height)));
                    tvHeight_user.invalidate();
                    tvWeight_user.setText((Float.toString(bodyData.weight)));
                    tvWeight_user.invalidate();
                    tvFat_user.setText((Float.toString(bodyData.fat)));
                    tvFat_user.invalidate();
                    tvMuscle_user.setText((Float.toString(bodyData.muscle)));
                    tvMuscle_user.invalidate();

                    //update graph
                    GraphView graph = (GraphView) findViewById(R.id.graph);
                    graph.getGridLabelRenderer().setPadding(32);

                    //create datapoints
                    DataPoint[] datapointWeight, datapointFat, datapointMuscle, datapointHeight;
                    datapointWeight = new DataPoint[bodyList.size()];
                    datapointFat = new DataPoint[bodyList.size()];
                    datapointMuscle = new DataPoint[bodyList.size()];
                    datapointHeight = new DataPoint[bodyList.size()];
                    int i = 0;
                    // set manual X bounds
                    StaticLabelsFormatter staticLabelsFormatter = new StaticLabelsFormatter(graph);
                    String[] Xlabels = new String[bodyList.size()];
                    for(BodyData bo: bodyList){
                        datapointWeight[i] = new DataPoint(i, bo.weight);
                        datapointFat[i] = new DataPoint(i, bo.fat);
                        datapointMuscle[i] = new DataPoint(i, bo.muscle);
                        datapointHeight[i] = new DataPoint(i, bo.height);
                        Xlabels[i] = ""+i;
                        i++;
                    }
                    LineGraphSeries<DataPoint> seriesWeight = new LineGraphSeries<>(datapointWeight);
                    LineGraphSeries<DataPoint> seriesFat = new LineGraphSeries<>(datapointFat);
                    LineGraphSeries<DataPoint> seriesMuscle = new LineGraphSeries<>(datapointMuscle);
                    LineGraphSeries<DataPoint> seriesHeight = new LineGraphSeries<>(datapointHeight);

                    seriesWeight.setDrawDataPoints(true);
                    seriesFat.setDrawDataPoints(true);
                    seriesMuscle.setDrawDataPoints(true);
                    seriesHeight.setDrawDataPoints(true);
                    if(Xlabels.length < 2) {
                        Xlabels = new String[2];
                        Xlabels[0] = "0";
                        Xlabels[1] = "1";
                        staticLabelsFormatter.setHorizontalLabels(Xlabels);
                    }else{
                        staticLabelsFormatter.setHorizontalLabels(Xlabels);
                    }
                    //add the lines
                    graph.getGridLabelRenderer().setLabelFormatter(staticLabelsFormatter);
                    graph.addSeries(seriesWeight);
                    graph.addSeries(seriesFat);
                    graph.addSeries(seriesMuscle);
                    //graph.addSeries(seriesHeight);

                    //legend
                    seriesWeight.setTitle("Weight(kg)");
                    seriesFat.setTitle("Fat %");
                    seriesMuscle.setTitle("Muscle %");
                    //seriesHeight.setTitle("Height(cm)");

                    //set colors
                    seriesWeight.setColor(Color.BLUE);
                    seriesFat.setColor(Color.GREEN);
                    seriesMuscle.setColor(Color.RED);
                    seriesHeight.setColor(Color.BLACK);

                    //set legend
                    graph.getLegendRenderer().setVisible(true);
                    graph.getLegendRenderer().setAlign(LegendRenderer.LegendAlign.TOP);
                    graph.invalidate();
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                Toast.makeText(getApplicationContext(), ERROR_READ, Toast.LENGTH_SHORT).show();
            }
        });
    }

    /**
     *Updates the recycleview with the list of subscribed workouts
     */
    protected void getSubWo(){
        checkConnection();
        //get subKeys from user
        //load workouts of those keys
        mUsers.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                for (DataSnapshot postSnapshot : dataSnapshot.getChildren()) {

                    User u = postSnapshot.getValue(User.class);

                    if(u.email.equals(getLoggedUser())){
                        subKeys = u.subKeys;
                        mWorkouts.addListenerForSingleValueEvent(new ValueEventListener() {
                            @Override
                            public void onDataChange(DataSnapshot dataSnapshot) {
                                ArrayList<WorkOut> woList = new ArrayList<WorkOut>();

                                for (DataSnapshot postSnapshot : dataSnapshot.getChildren()) {
                                    WorkOut wo = postSnapshot.getValue(WorkOut.class);
                                    if(subKeys.contains(wo.key)) {
                                        woList.add(wo);
                                    }
                                }
                                //firebase finished

                                //UPDATE RECYCLEVIEW

                                RecyclerView recyclerView = (RecyclerView) findViewById(R.id.recyclerview);
                                WoAdapter mAdapter;
                                mAdapter= new WoAdapter(woList, CommonActivity.this);
                                RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(getApplicationContext());
                                recyclerView.setLayoutManager(mLayoutManager);
                                recyclerView.setItemAnimator(new DefaultItemAnimator());
                                recyclerView.setAdapter(mAdapter);
                                recyclerView.invalidate();
                            }

                            @Override
                            public void onCancelled(DatabaseError databaseError) {
                                Toast.makeText(getApplicationContext(), ERROR_READ, Toast.LENGTH_SHORT).show();
                            }
                        });
                    }
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                Toast.makeText(getApplicationContext(), ERROR_READ, Toast.LENGTH_SHORT).show();
            }
        });




    }
    /**
     * Obtain all the workouts from firebase
     * @param size
     * @param newest
     */
    protected void getAllWorkOuts(final int size, final boolean newest){
        checkConnection();
        mWorkouts.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                ArrayList<WorkOut> woList = new ArrayList<WorkOut>();

                for (DataSnapshot postSnapshot : dataSnapshot.getChildren()) {
                    WorkOut wo = postSnapshot.getValue(WorkOut.class);
                    woList.add(wo);
                }

                if(size != -1) {//RANKING MODE
                    //SORT FROM HIGH SCORE TO LOW SCORE
                    Collections.sort(woList, new Comparator<WorkOut>() {
                        @Override
                        public int compare(WorkOut z1, WorkOut z2) {
                            if (z1.score > z2.score)
                                return -1;
                            if (z1.score < z2.score)
                                return 1;
                            return 0;
                        }
                    });
                    //pick top 10
                    if(woList.size() > size)
                        woList = new ArrayList<WorkOut>(woList.subList(0,10));
                }else{
                    if(newest)
                        Collections.reverse(woList);
                }
                //firebase finished
                //UPDATE SOMETHING

                RecyclerView recyclerView = (RecyclerView) findViewById(R.id.recyclerview);
                WoAdapter mAdapter;
                mAdapter= new WoAdapter(woList, CommonActivity.this);
                RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(getApplicationContext());
                recyclerView.setLayoutManager(mLayoutManager);
                recyclerView.setItemAnimator(new DefaultItemAnimator());
                recyclerView.setAdapter(mAdapter);
                recyclerView.invalidate();
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                Toast.makeText(getApplicationContext(), ERROR_READ, Toast.LENGTH_SHORT).show();
            }
        });
    }

    /**
     * Rates the workout equivalent of the wo_key
     * @param rate
     * @param wo_key
     */
    protected void rateWorkout(final float rate, final String wo_key){
        checkConnection();
        mWorkouts.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                for (DataSnapshot postSnapshot : dataSnapshot.getChildren()) {
                    WorkOut wo = postSnapshot.getValue(WorkOut.class);
                    if(wo.key.equals(wo_key)) {
                        //RATE THE WORKOUT
                        Map<String, Object> scoreUpdate = new HashMap<String, Object>();
                        Map<String, Object> votesUpdate = new HashMap<String, Object>();
                        Map<String, Object> totalScoreUpdate = new HashMap<String, Object>();
                        //compute the mean score
                        wo.votes++;
                        wo.totalScore += rate;
                        wo.score = wo.totalScore / wo.votes;
                        //put the fields to update
                        scoreUpdate.put("score", wo.score);
                        scoreUpdate.put("votes", wo.votes);
                        scoreUpdate.put("totalScore", wo.totalScore);
                        //update childs
                        mWorkouts.child(wo.key).updateChildren(scoreUpdate);
                        mWorkouts.child(wo.key).updateChildren(votesUpdate);
                        mWorkouts.child(wo.key).updateChildren(totalScoreUpdate);

                        Toast.makeText(getApplicationContext(), "Workout rated with "+rate+" stars!", Toast.LENGTH_SHORT).show();
                    }

                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                Toast.makeText(getApplicationContext(), ERROR_READ, Toast.LENGTH_SHORT).show();
            }
        });
    }

    /**
     * updates buttons if subscribed/not subscribed
     * @param email
     * @param wo_key
     */
    protected void check_sub(final String email, final String wo_key){
        checkConnection();
        mUsers.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                for (DataSnapshot postSnapshot : dataSnapshot.getChildren()) {

                    User u = postSnapshot.getValue(User.class);

                    if(u.email.equals(email)){
                        Button subs_btn = (Button) findViewById(R.id.subs_btn);
                        Button unsubs_btn = (Button) findViewById(R.id.unsubs_btn);
                        if(subs_btn != null && unsubs_btn!= null) {
                            if (u.subKeys.contains(wo_key)) {
                                //already sub
                                //disable subscribe button

                                subs_btn.setVisibility(View.GONE);
                                unsubs_btn.setVisibility(View.VISIBLE);
                            } else {
                                //not sub
                                //disable unsubscribe button
                                subs_btn.setVisibility(View.VISIBLE);
                                unsubs_btn.setVisibility(View.GONE);
                            }
                            subs_btn.invalidate();
                            unsubs_btn.invalidate();
                        }

                    }
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                Toast.makeText(getApplicationContext(), ERROR_READ, Toast.LENGTH_SHORT).show();
            }
        });
    }
    /**
     * Subscribes the user to a workout
     * @param email
     * @param wo_key
     */
    protected void subscribe(final String email, final String wo_key){
        //subscribe user email to workout key
        //get key user
        checkConnection();
        mUsers.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                for (DataSnapshot postSnapshot : dataSnapshot.getChildren()) {

                    User u = postSnapshot.getValue(User.class);

                    if(u.email.equals(email)){

                        //add wo_key to subkeys
                        if(!u.subKeys.contains(wo_key)) {
                            u.subKeys.add(wo_key);
                            //update user
                            Map<String, Object> subUpdate = new HashMap<String, Object>();
                            subUpdate.put("subKeys", u.subKeys);
                            mUsers.child(postSnapshot.getKey()).updateChildren(subUpdate);
                            Toast.makeText(getApplicationContext(), "Subscribed to the workout!", Toast.LENGTH_SHORT).show();
                            check_sub(email,wo_key);


                        }else{
                            Toast.makeText(getApplicationContext(), "Already subscribed!", Toast.LENGTH_SHORT).show();
                        }
                    }
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                Toast.makeText(getApplicationContext(), ERROR_READ, Toast.LENGTH_SHORT).show();
            }
        });

    }
    protected void unsubscribe(final String email, final String wo_key){
        //unsubscribe user email from workout key
        //get key user
        checkConnection();
        mUsers.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                for (DataSnapshot postSnapshot : dataSnapshot.getChildren()) {

                    User u = postSnapshot.getValue(User.class);

                    if(u.email.equals(email)){

                        //remove wo_key from subkeys
                        if(u.subKeys.contains(wo_key)) {
                            u.subKeys.remove(wo_key);
                            //update user
                            Map<String, Object> subUpdate = new HashMap<String, Object>();
                            subUpdate.put("subKeys", u.subKeys);
                            mUsers.child(postSnapshot.getKey()).updateChildren(subUpdate);
                            Toast.makeText(getApplicationContext(), "Unsuscribed from the workout!", Toast.LENGTH_SHORT).show();
                            check_sub(email,wo_key);
                        }
                    }
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                Toast.makeText(getApplicationContext(), ERROR_READ, Toast.LENGTH_SHORT).show();
            }
        });

    }
    /**
     * Changes the font of the title activity
     */
    protected void setFonts(){
        //SET FONT
        Typeface tf = Typeface.createFromAsset(getAssets(),
                "fonts/spacejunkXL.ttf");
        TextView tv = (TextView) findViewById(R.id.tittle_name);
        tv.setTypeface(tf);
        TextView tv2 = (TextView) findViewById(R.id.textViewTitle);
        tv2.setTypeface(tf);
    }

    /**
     * Loads the activity layout in the view
     * @param idLayout
     */
    protected void changeLayoutActivity(int idLayout){
        //we change the include_layout to include the current activity xml
        LinearLayout incl_layout = (LinearLayout) findViewById(R.id.include_layout);
        LayoutInflater inflater = (LayoutInflater)getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View currentActivity = inflater.inflate(idLayout, null);
        incl_layout.removeAllViews();
        incl_layout.addView(currentActivity);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();


        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onBackPressed() {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            finish();
        }
    }

    /**
     * Logs out from the app and returns to login screen
     */
    public void logOut(){
        FirebaseAuth.getInstance().signOut();
        startActivity(new Intent(this, LoginActivity.class));
        finish();
    }

    /**
     * Retrieves the current user logged in firebase
     * @return
     */
    protected String getLoggedUser(){
        FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
        if (user != null) {
            return user.getEmail();
        }

        return "";
    }


    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(MenuItem item) {
        // Handle navigation view item clicks here.
        int id = item.getItemId();

        if (id == R.id.nav_createwo) {
            if(!this.getClass().getName().equals(NewWOActivity.class.getName())){
                startActivity(new Intent(this, NewWOActivity.class));
                finish();
            }
        } else if (id == R.id.nav_mywo) {
            if(!this.getClass().getName().equals(MyWOActivity.class.getName())) {
                startActivity(new Intent(this, MyWOActivity.class));
                finish();
            }
        } else if (id == R.id.nav_ranking) {
            if(!this.getClass().getName().equals(RankingActivity.class.getName())) {
                startActivity(new Intent(this, RankingActivity.class));
                finish();
            }
        } else if (id == R.id.nav_bdata) {
            if(!this.getClass().getName().equals(BodyDataActivity.class.getName())) {
                startActivity(new Intent(this, BodyDataActivity.class));
                finish();
            }
        }else if (id == R.id.nav_findwo) {
            if(!this.getClass().getName().equals(LookAllWOActivity.class.getName())) {
                startActivity(new Intent(this, LookAllWOActivity.class));
                finish();
            }
        }else if (id == R.id.nav_logout) {
            logOut();
        }

        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        drawer.closeDrawer(GravityCompat.START);
        return true;
    }

    /**
     * Recover the workout clicked in the recycleView and start the WODetailActivity passing the workout in the bundle
     * @param wo
     */
    public void recycle_click(WorkOut wo){
        Intent i = new Intent();
        Bundle b = new Bundle();
        b.putSerializable(BUNDLE_NAME, wo);
        i.putExtras(b);
        i.setClass(this,WODetailActivity.class);
        startActivity(i);
        finish();
    }

}
