package com.juanjo.uoc.workoutshare;

import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.RatingBar;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.io.FileOutputStream;

public class WODetailActivity extends CommonActivity {
    private String wo_key;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //we change the include_layout to include the current activity xml
        changeLayoutActivity(R.layout.activity_wodetail);
        setFonts();
        loadWorkout();

        Button unsubs_btn = (Button) findViewById(R.id.unsubs_btn);
        unsubs_btn.setVisibility(View.GONE);
        Button subs_btn = (Button) findViewById(R.id.subs_btn);
        subs_btn.setVisibility(View.VISIBLE);
        Button share_btn = (Button) findViewById(R.id.share_btn);

        Button rate_btn = (Button) findViewById(R.id.rate_btn);
        rate_btn.setOnClickListener(new View.OnClickListener(){
            public void onClick(View v) {
                //RATE
                RatingBar mBar = (RatingBar) findViewById(R.id.ratingBar);

                rateWorkout(mBar.getRating(),wo_key);
            }
        });
        //CHECK IF SUBSCRIBED
        check_sub(getLoggedUser(),wo_key);


        subs_btn.setOnClickListener(new View.OnClickListener(){
            public void onClick(View v) {
                //SUBSCRIBE
                subscribe(getLoggedUser(), wo_key);
            }
        });

        unsubs_btn.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                //UNSUBSCRIBE
                unsubscribe(getLoggedUser(), wo_key);
            }
        });
        share_btn.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                //SHARE WORKOUT
                View rootView = getWindow().getDecorView().findViewById(android.R.id.content);
                Bitmap bitmap = getScreenShot(rootView);
                store(bitmap, "shared_workout.png");
                final String dirPath = Environment.getExternalStorageDirectory().getAbsolutePath() + "/Screenshots";
                File file = new File(dirPath, "shared_workout.png");
                shareImage(file);
            }
        });


    }

    /**
     * Load textviews, ratings in the activity with the workout info
     */
    public void loadWorkout(){
        WorkOut wo;
        Bundle b = this.getIntent().getExtras();
        TextView title = (TextView) findViewById(R.id.textViewTitle);
        TextView descr = (TextView) findViewById(R.id.wo_descr);
        RatingBar mBar = (RatingBar) findViewById(R.id.ratingBar);
        if (b != null) {
            wo = (WorkOut) b.getSerializable(BUNDLE_NAME);
            title.setText(wo.name);
            wo_key = wo.key;
            mBar.setRating(wo.score);
            descr.setText(wo.description);
        }

    }

    /**
     * creates a screenshot bitmap
     * @param view
     * @return
     */
    public static Bitmap getScreenShot(View view) {
        View screenView = view.getRootView();
        screenView.setDrawingCacheEnabled(true);
        Bitmap bitmap = Bitmap.createBitmap(screenView.getDrawingCache());
        screenView.setDrawingCacheEnabled(false);
        return bitmap;
    }

    /**
     * stores the bitmap with filename name
     * @param bm
     * @param fileName
     */
    public static void store(Bitmap bm, String fileName){
        final String dirPath = Environment.getExternalStorageDirectory().getAbsolutePath() + "/Screenshots";
        File dir = new File(dirPath);
        if(!dir.exists())
            dir.mkdirs();

        File file = new File(dirPath, fileName);
        try {
            FileOutputStream fOut = new FileOutputStream(file);
            bm.compress(Bitmap.CompressFormat.PNG, 85, fOut);
            fOut.flush();
            fOut.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * shares the file to another app
     * @param file
     */
    private void shareImage(File file){
        Uri uri = Uri.fromFile(file);
        Intent intent = new Intent();
        intent.setAction(Intent.ACTION_SEND);
        intent.setType("image/*");

        intent.putExtra(android.content.Intent.EXTRA_SUBJECT, "");
        intent.putExtra(android.content.Intent.EXTRA_TEXT, "");
        intent.putExtra(Intent.EXTRA_STREAM, uri);
        try {
            startActivity(Intent.createChooser(intent, "Share Screenshot"));
        } catch (ActivityNotFoundException e) {
            Toast.makeText(getApplicationContext(), "No App Available", Toast.LENGTH_SHORT).show();
        }
    }
}
